% this script analysis the perception data of enhanced speech.
%
% The SNRLoss07EN experiment is focused on the two sounds, /ka/s and /ga/s, that have been shown to be problematic for Annette
% by previous experiment. The purpose of the experiment is to see whether we can improve the recognition score for these two 
% consonants by manipulating the features. The experiment has been conducted for multiple rounds:
%
% Round-1: SNRLoss07EN1
% The /ka/s and /ga/s are boosted in the feature area by factors of [0, 1, 10, 50] with and without NAL-R; It turns out that 
% the speech are distorted too much due to the too-big boost factors. As a consequence, the subject had a score significantly 
% lower for the enhanced speech than the original speech sounds.
%
% Round-2: SNRLoss07EN2
% The /ka/s and /ga/s are boosted in the feature area by factors of [1, 2, 4, 6] with NAL-R. The subject show slight improvement 
% under quiet condition, no difference at 12 dB SNR.
%
% Round-3: SNRLoss07RM2
% Previous results show that the subject has some strong patterns of confusions, such as /ka/ to /ta/ and /ga/ to /da/. 
% To compensate for that in this experiment we remove the high-frequency region in /ka/s and /ga/s that cause the afore-mentioned 
% morphing of /ta/ and /da/.
% 
% Round-4: SNRLoss07RE1
% This experiment combines the round-2 and round-3 techniques, i.e, removing /ta/ or /da/ cues in /ka/ and /ga/ and boosting the 
% /ka/, /ga/ features.

% Round-5: SNRLoss07SW
% In the previous experiment, we found that the HI listener's PI functions for a single consonant sound varies a lot for different 
% talkers. In this experiment we aim to find out all the natural strong /ka/s and /ga/s.

% For Round1-4 there are two ears (AS-L and AS-R). For Round-5, there are
% four ears AS-L and AS-R for ptkbdg, and DC-L and DC-R for fsSvzZ

close all; clear all; clc;

load SPDB_SL07EN.mat; % for detail of the 

nSNR = length(Index_SL07EN.snr);
nSub = length(Index_SL07EN.listener);

%% create CM for the first round: SNRLoss07EN1
stims = {'pa','ta','ka','ka_0','ka_10','ka_50','ba','da','ga','ga_0','ga_10','ga_50'}; % 10, 50 proved to be too big a gain
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

for k=1:2
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        
        % ---- calculate CM for the data without NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN', 'comment', 'EN1' );	% retrieve data at a given snr

        if ~isempty(data{1})
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM = calc_CM_en(data, stims, resps, ['SNRLoss07EN1-No-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            %cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document                
            tab_mini = CM.table(:,[1,2,3,8,9,10]); resps_mini = CM.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM.title '.tex'], CM.stims, resps_mini); % output to Latex document    
        end
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'comment', 'EN1' );	% retrieve data at a given snr

        if ~isempty(data{1})        
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM = calc_CM_en(data, stims, resps, ['SNRLoss07EN1-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            %cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
            tab_mini = CM.table(:,[1,2,3,8,9,10]); resps_mini = CM.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM.title '.tex'], CM.stims, resps_mini); % output to Latex document    
        end
        
    end
end

%% create CM for the second round: SNRLoss07EN2
stims = {'pa','ta','ka','ka_2','ka_4','ka_6','ba','da','ga','ga_2','ga_4','ga_6'}; % 2, 4, 6 could be better
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

for k=1:2
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'comment', 'EN2' );	% retrieve data at a given snr

        if ~isempty(data{1})                
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM = calc_CM_en(data, stims, resps, ['SNRLoss07EN2-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            %cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
            tab_mini = CM.table(:,[1,2,3,8,9,10]); resps_mini = CM.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM.title '.tex'], CM.stims, resps_mini); % output to Latex document    
        end
        
    end
end

%% create CM for the third round: SNRLoss07RM1
stims = {'pa','ta','ka','ka_0','ba','da','ga','ga_0'}; % ka_0 means the /ta/ feature in /ka/ removed, same for ga_0
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

for k=1:2
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'comment', 'RM1' );	% retrieve data at a given snr

        if ~isempty(data{1})                
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM = calc_CM_en(data, stims, resps, ['SNRLoss07RM1-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            %cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
            tab_mini = CM.table(:,[1,2,3,8,9,10]); resps_mini = CM.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM.title '.tex'], CM.stims, resps_mini); % output to Latex document    
        end
    end
end

%% create CM for the forth round: SNRLoss07RE1
% ka_1 means the /ta/ feature in /ka/ removed, same for ga_1; ka_2 means
% /ta/ feature removed plus /ka/ feature boosted by 2 (6dB); ka_4 means
% /ta/ feature removed plus /ka/ feature boosted by 4 (12dB).
stims = {'pa','ta','ka','ka_1','ka_2','ka_4','ba','da','ga','ga_1','ga_2','ga_4'}; 
resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'}; 

for k=1:2
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        
        % ---- calculate CM for the data with NAL-R ----
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'comment', 'RE1' );	% retrieve data at a given snr

        if ~isempty(data{1})        
            % replace stims with utter for the enhanced speech, for example, ka to ka_10;
            for j=1:length(data{11})
                p = strfind(data{2}{j}, '_');
                data{11}{j} = data{2}{j}(p(2)+1:end);
            end
        
            CM = calc_CM_en(data, stims, resps, ['SNRLoss07RE1-NAL-R-SNR=' num2str(cur_snr) 'dB_' cur_sub]);  % calculate confusion matrix
            %cm2tex(CM.table, ['CM' CM.title '.tex'], CM.stims, CM.resps); % output to Latex document    
            tab_mini = CM.table(:,[1,2,3,8,9,10]); resps_mini = CM.resps([1,2,3,8,9,10]);
            cm2tex(tab_mini, ['CM_mini_' CM.title '.tex'], CM.stims, resps_mini); % output to Latex document    
        end
        
    end
end


%% compare the score of /ka/s and /ga/s from six talkers and find out which 
% are the /ka/s and /ga/s that the subject can hear. consider the original
% sounds only. For AS-L and AS-R 

% kas = { 'm111','f103','m118','f105','m114','f119'};
% gas = { 'm111','f119','m104','f108','m112','f109'};  

kas = {'f101','f103','f105','f106','f108','f109','f113','f119','m102',...
       'm104','m107','m111','m112','m114','m115','m117','m118','m120'};

gas = {'f101','f103','f105','f106','f108','f109','f113','f119','m102',...
       'm104','m107','m111','m112','m114','m115','m117','m118','m120'};
 
h = figure('Position', [50, 50, 800, 300]); set(h,'PaperPositionMode','auto');

for k=1:2
    
    pi_ka_noNAL = zeros(length(kas),2); % pi of /ka/s from 6 talkers at 12 and 100dB SNR
    pi_ga_noNAL = zeros(length(gas),2); % pi of /ga/s from 6 talkers at 12 and 100dB SNR
   
    pi_ka_NAL = zeros(length(kas),2); % pi of /ka/s from 6 talkers at 12 and 100dB SNR
    pi_ga_NAL = zeros(length(gas),2); % pi of /ga/s from 6 talkers at 12 and 100dB SNR    
    
    cur_sub = Index_SL07EN.listener{k};
    for i=1:nSNR
        cur_snr = Index_SL07EN.snr(i);
        for j=1:length(kas)
            cur_utter = ['s_' kas{j} '_ka'];  % ka
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN', 'utter', cur_utter);
            pi_ka_noNAL(j,i) = sum(data{23})/(length(data{23})+eps);
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'utter', cur_utter);
            pi_ka_NAL(j,i) = sum(data{23})/(length(data{23})+eps);
        end

        clf;
        bar1=bar([1:length(kas)], pi_ka_NAL(:,i), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
        bar2=bar([1:length(kas)], pi_ka_noNAL(:,i), 'FaceColor', 'b', 'EdgeColor', 'b');
        set(bar1,'BarWidth',0.4); set(bar2,'BarWidth',0.2); legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
        xlim([0, length(kas)+1]); ylim([0, 1.05]); 
        %title('Pc of /ka/s from different talkers for AS-L at 12 dB SNR', 'FontSize', 12);
        xlabel('Talker', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
        set(gca,'XTick',[1:length(kas)], 'XTickLabel', kas, 'FontSize', 9); 
        set(gca,'YTick',[0:0.2:1], 'FontSize', 9);
        %saveas(gcf, 'Pc_kas_AS-L_12dB.eps', 'psc2');
        print(h,'-depsc2', ['Pc_kas_' cur_sub '_' num2str(cur_snr) 'dB.eps']);
           
        for j=1:length(gas)
            cur_utter = ['s_' gas{j} '_ga'];  % ga
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NaN', 'utter', cur_utter);
            pi_ga_noNAL(j,i) = sum(data{23})/(length(data{23})+eps);            
            data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', cur_snr, 'compensate', 'NAL-R', 'utter', cur_utter);
            pi_ga_NAL(j,i) = sum(data{23})/(length(data{23})+eps);            
        end
        
        clf;
        bar1=bar([1:length(gas)], pi_ga_NAL(:,i), 'FaceColor', 'w', 'EdgeColor', 'b'); hold on;
        bar2=bar([1:length(gas)], pi_ga_noNAL(:,i), 'FaceColor', 'b', 'EdgeColor', 'b');
        set(bar1,'BarWidth',0.4); set(bar2,'BarWidth',0.2); legend({'NAL-R'; 'No NAL-R'}, 'Location', 'NorthEast');
        xlim([0, length(gas)+1]); ylim([0, 1.05]); 
        %title('Pc of /ga/s from different talkers for AS-L at 100 dB SNR', 'FontSize', 12);
        xlabel('Talker', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
        set(gca,'XTick',[1:length(kas)], 'XTickLabel', kas, 'FontSize', 9); 
        set(gca,'YTick',[0:0.2:1], 'FontSize', 9);
        %saveas(gcf, 'Pc_gas_AS-L_12dB.eps', 'psc2');
        print(h,'-depsc2', ['Pc_gas_' cur_sub '_' num2str(cur_snr) 'dB.eps']);        
        
    end
    
end


%% compare the score of /fa/s and /va/s from 18 talkers and find out which 
% are the /ka/s and /ga/s that the subject can hear. consider the original
% sounds only. for DC-L and DC-R

fas = {'f101','f103','f105','f106','f108','f109','f113','f119','m102',...
       'm104','m107','m111','m112','m114','m115','m117','m118','m120'};

vas = {'f101','f103','f105','f106','f108','f109','f113','f119','m102',...
       'm104','m107','m111','m112','m114','m115','m117','m118','m120'};

listeners = {'DC-L'; 'DC-R'};

pi_fa_NAL = zeros(length(listeners), length(fas)); % pi of /fa/s from 18 talkers at 100dB SNR
pi_va_NAL = zeros(length(listeners), length(vas)); % pi of /va/s from 18 talkers at 100dB SNR    

for k=1:length(listeners)
    
    cur_sub = listeners{k};

    for j=1:length(fas)
        cur_utter = ['s_' fas{j} '_fa'];  % fa
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', 100, 'compensate', 'NAL-R', 'utter', cur_utter);
        pi_fa_NAL(k,j) = sum(data{23})/10;
    end

    clf;
    bar1=bar([1:length(fas)], pi_fa_NAL(k,:), 'FaceColor', 'b', 'EdgeColor', 'b');
    set(bar1,'BarWidth',0.4); xlim([0, length(fas)+1]); ylim([0, 1.05]); 
    %title('Pc of /fa/s from different talkers for DC-L at 100 dB SNR with NAL-R', 'FontSize', 12);
    xlabel('Talker', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
    set(gca,'XTick',[1:length(fas)], 'XTickLabel', fas, 'FontSize', 9); 
    set(gca,'YTick',[0:0.2:1], 'FontSize', 9);
    %saveas(gcf, 'Pc_fas_DC-L_100dB.eps', 'psc2');
    print(h,'-depsc2', ['Pc_fas_' cur_sub '_100dB.eps']);
    
    for j=1:length(vas)
        cur_utter = ['s_' vas{j} '_va'];  % va
        data = query(SPDB_SL07EN, 'listener', cur_sub, 'snr', 100, 'compensate', 'NAL-R', 'utter', cur_utter);
        pi_va_NAL(k,j) = sum(data{23})/10;            
    end

    clf;
    bar1=bar([1:length(vas)], pi_va_NAL(k,:), 'FaceColor', 'b', 'EdgeColor', 'b');
    set(bar1,'BarWidth',0.4); xlim([0, length(vas)+1]); ylim([0, 1.05]); 
    %title('Pc of /va/s from different talkers for DC-L at 100 dB SNR with NAL-R', 'FontSize', 12);
    xlabel('Talker', 'FontSize', 12); ylabel('Percent Correct', 'FontSize', 12);
    set(gca,'XTick',[1:length(fas)], 'XTickLabel', fas, 'FontSize', 9); 
    set(gca,'YTick',[0:0.2:1], 'FontSize', 9);
    %saveas(gcf, 'Pc_vas_DC-L_100dB.eps', 'psc2');
    print(h,'-depsc2', ['Pc_vas_' cur_sub '_100dB.eps']);
    
end
