%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program calculate the Confusion Pattern for a given set of data as a 
% function of filtering, truncation, snr conditions.
% Input:    data - speech perception data of an particular utterance to be analyzed
%           caption - the utterance to be analyzed
%           type - time, frequency or snr
% Output:   CP - confusion pattern as a function of time, frequency
%                or snr condition.
%   Author:      Feipeng Li at HSR, Beckman Institute, UIUC
%   Time-stamp:  2007-11-28
%   E-mail:      fli2@uiuc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function CP = calc_CP(data, caption, type)

CP = NaN;

% % check consistency between data and utter (records of utterances other than utter can not be processed)
% utter_in_data = unique(sort(data{2}));
% 
% if length(utter_in_data)>1
%     disp('genCP can process only one utterance at a time!');
%     return;
% end
% 
% if ~strcmp(utter_in_data{1}, utter)
%     disp('the data and utter are not consistent with each other!');
%     return;
% end

switch lower(strtrim(type))
    case 'tr08'
        col_no = 13; xlab = 'Truncation time [cs]'; 
        resps = {'p','t','k','f','xt','s','xs','b','d','g','v','xd','z','xz','m','n','?','.'};         
    case 'time'
        col_no = 13; xlab = 'Truncation time [cs]'; 
        resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?','a'};         
    case 'lpf08'
        col_no = 14; xlab = 'Low-pass cutoff frequency (Hz)';
        resps = {'p','t','k','f','xt','s','xs','b','d','g','v','xd','z','xz','m','n','?'};         
    case 'hpf08'
        col_no = 14; xlab = 'High-pass cutoff frequency (Hz)';
        resps = {'p','t','k','f','xt','s','xs','b','d','g','v','xd','z','xz','m','n','?'};    
    case 'lpf'
        col_no = 14; xlab = 'Low-pass cutoff frequency (Hz)';
        resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'};         
    case 'hpf'
        col_no = 14; xlab = 'High-pass cutoff frequency (Hz)';
        resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'};                 
    case 'snr'
        col_no = 15; xlab = 'SNR (dB)';
        resps = {'pa','ta','ka','fa','xta','sa','xsa','ba','da','ga','va','xda','za','xza','ma','na','?'};  
    case 'cv06wn'
        col_no = 15; xlab = 'SNR (dB)';        
        Cs = {'p', 't', 'k', 'd', 'g', 's', 'xs', 'z', 'xz'};
        Vs = {'a', 'o', 'u', 'xe', 'xi', 'xq', 'xr', 'xu'};
        k = 1;
        for i=1:length(Cs)
            for j=1:length(Vs)
                resps{k} = [Cs{i} Vs{j}]; k = k+1;                
            end
        end
    otherwise
        disp('Invalid parameters! type = time, lpf/hpf or snr');
        return
end

ylab = 'Num of Response/Num of Presentation';

% replace Resp with their row and colume no for fast processing. Those
% responses not included in Resps, for example, xja, are attributed to be ?
column_pos = zeros(size(data{20})); 
for j=1:length(resps) % convert Resp to an address in Table
    p = find(strcmp(data{20}, resps{j}));
    column_pos(p) = j;
end
if strcmp(type, 'cv06wn')
    column_pos = floor((column_pos-1)/length(Vs))+1;
    resps = Cs; resps{end+1} = '?';
    p = find(column_pos==0);   
    column_pos(p) = length(Vs)+1;    
else
    p = find(column_pos==0);   
    column_pos(p) = 17;    
end

% get all the conditions (truncation, filterint, snr) associated with the
% utterance.
conditions = unique(sort(data{col_no}));

if length(conditions)>50
    disp('too many conditions! something might be wrong!');
    return;
end

row_pos = zeros(size(data{col_no}));
for i=1:length(conditions)
    p = find(data{col_no}==conditions(i));
    row_pos(p) = i;
end

p = find(row_pos==0);
if ~isempty(p)
    disp('there is something wrong in establishing hash table!');
    return;
end

% the confusion pattern table 
table = zeros(length(conditions), length(resps));
nrecord = length(data{1});
for i=1:nrecord
    table(row_pos(i),column_pos(i)) = table(row_pos(i),column_pos(i)) + 1;
end

nPresent = sum(table, 2);
p = find(nPresent==0);
table(p,:) = NaN;

clear CP;

if (col_no==13)     % truncation time
    conditions = round(unique(sort(data{col_no}))*1000)/10;
end

CP.caption = caption;
CP.resps = resps;
CP.conds = conditions;
CP.xlabel = xlab;
CP.ylabel = ylab;
CP.table = table;
CP.type = type;


