% This program calculate the time, frequency, and snr importance 
% functions based on TR07, HL07, and MN16R data.

function calc_importance_aigram_rev(SPDB, utter, vow_start_time)

cv = utter(6:end);

%% calculate the probability error of the distorted speech
% calculate Pe as a function of truncation time
data = query(SPDB, 'exp', 'TRUNC07', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'time');
eT = pe_tem.score;  % probability error of truncated utterance
t = pe_tem.conds;   % truncation time
cp_time = calc_CP(data, utter, 'time');

% calculate Pe as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
pe_tem = calc_Pe(data, cv, utter, 'hpf');
eH = pe_tem.score;  % probability error of high-pass filtered speech
fH = pe_tem.conds;  % high-pass cutoff frequency
cp_H = calc_CP(data, utter, 'hpf');

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
pe_tem = calc_Pe(data, cv, utter, 'lpf');
eL = pe_tem.score;  % probability error of low-pass filtered speech
fL = pe_tem.conds;  % low-pass cutoff frequency
cp_L = calc_CP(data, utter, 'lpf');

% calculate Pe as a function of signal-to-noise ratio
data = query(SPDB, 'exp', 'MN16R', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'snr');
eSNR = pe_tem.score;
snrs = pe_tem.conds;
snrs(end) = 24;
cp_snr = calc_CP(data, utter, 'snr');
 
%% calculate the importance function from MN16R data
emin = max(min(eSNR),0.02);
p = find(eSNR<emin); eSNR(p) = emin;
ai_snr = log10(eSNR)/log10(emin);
dSNR = (ai_snr(2:end)-ai_snr(1:end-1))./(snrs(2:end)-snrs(1:end-1));
dSNR = [dSNR(1); dSNR];
dSNR = dSNR/(sum(dSNR));    % normalize the coefficients

importance_snr.snrs = snrs;
importance_snr.dSNR = dSNR; %importance_snr.dSNR = dSNR.*(1-eSNR);

if min(eSNR)<0.1
    p1 = max(find(eSNR>0.1)); if (isnan(p1)), p1=1; end
    p2 = min(find(eSNR<0.1)); if (isnan(p2)), p2=length(snrs); end
    SNR90 = interp1(eSNR([p1,p2]), snrs([p1,p2]), 0.10, 'linear', 'extrap');
else
    SNR90 = NaN;
end

%% find the snr where the dominant feature is still available
p = find(dSNR==max(dSNR(5:end-1)));
snr_star = snrs(max(p)); 
snr = 18;

%% calculate frequency importance functions
% interplate eL and eH across all the frequencies
fLH = unique(sort([fL; fH]));

% preprocessing, get rid of the effect of interfering cue and make sure the
% eL and eH are monotonically increasing or decreasing
for i=2:length(eL)
    eL(i) = min(eL(i-1), eL(i));
end
for i=length(eH)-1:-1:1
    eH(i) = min(eH(i+1), eH(i));
end

eL = [max(15/16, eL(1)); eL]; fL = [fH(1); fL];
eH = [eH; max(15/16, eH(end))]; fH = [fH; fL(end)];

eL = interp1(fL, eL, fLH, 'linear');
eH = interp1(fH, eH, fLH, 'linear');

% get rid of eL and eH that are smaller than chance level
emin = max(min(min(eL),min(eH)),0.02);
p = find(eL<emin); eL(p) = emin;
p = find(eH<emin); eH(p) = emin;

% find the distance of the cutoff frequencies on BM
x1 = greenwood_inverse(184.4644);  % lowest frequency covered in AIgram
x2 = greenwood_inverse(7397.3);    % highest frequency covered in AIgram
xAI = linspace( x1, x2, 200);      % distances of 200 bands in AI-gram
fAI = greenwood_inverse(xAI, 'dist');

% convert the cut-off frequecies into auditory filter band number
xLH = greenwood_inverse(fLH);      % distances on BM that corresponds to fL and fH
for i=1:length(xLH)
    temp = abs(xAI-xLH(i));
    band_AI(i) = find(temp==min(temp));              % the AI band no corresponds to the cut-off frequency
end

% calculate the frequency importance function
dL = NaN(1, length(fLH)); 
dH = NaN(1, length(fLH)); 
dL(2:end) = log10(eL(2:end))/log10(emin) - log10(eL(1:end-1))/log10(emin);    % lpf weights ekL
dH(1:end-1) = log10(eH(1:end-1))/log10(emin) - log10(eH(2:end))/log10(emin);  % hpf weights ekH
dL(1) =  dH(1);  dH(end) =  dL(end);
p = find(dL<0); dL(p)=0;
p = find(dH<0); dH(p)=0;
dL = dL/sum(dL); dH = dH/sum(dH);    % normalize the importance function
%dL = dL.*(1-eL'); dH = dH.*(1-eH');

% save the data of HL07 in a struct
pe_freq.eL = eL; 
pe_freq.eH = eH;
pe_freq.dL = dL; 
pe_freq.dH = dH;
pe_freq.f = fLH;
pe_freq.x = xLH;
pe_freq.bands = band_AI;

importance_freq.x = xAI;
importance_freq.f = fAI;
importance_freq.bands = [1:length(xAI)];

% % method 1 for the integration of dL and dH
% importance_freq.dL = interp1(xLH, dL, xAI, 'linear', 'extrap');
% importance_freq.dH = interp1(xLH, dH, xAI, 'linear', 'extrap');
% importance_freq.dLH = (importance_freq.dL+importance_freq.dH)/2; % method1 for the integration of dL and dH

% method 2 for the integration of dL and dH
for i=1:length(dL)
    dLH(i) = max(dL(i), dH(i)); % method2 for the integration of dL and dH
end
importance_freq.dL = interp1(xLH, dL, xAI, 'linear', 'extrap');
importance_freq.dH = interp1(xLH, dH, xAI, 'linear', 'extrap');
importance_freq.dLH = interp1(xLH, dLH, xAI, 'linear', 'extrap');

% % method 3 for the integration of dL and dH
% err = eL - eH; 
% err_product = err(1:end-1).*err(2:end); 
% p = max(find(err_product<=0)); % the position where the sign of err changes
% if abs(err(p))>abs(err(p+1))
%     band_X = p+1;
% else
%     band_X = p;
% end
% dLH = zeros(size(fLH));
% dLH(1:band_X) = dL(1:band_X);
% dLH(band_X+1:end) = dH(band_X+1:end);
% importance_freq.dL = interp1(xLH, dL, xAI, 'linear', 'extrap');
% importance_freq.dH = interp1(xLH, dH, xAI, 'linear', 'extrap');
% importance_freq.dLH = interp1(xLH, dLH, xAI, 'linear'); % method3 for the integration of dL and dH
% p = find(importance_freq.dLH<0); importance_freq.dLH(p) = 0;

% % method 4 for the integration of dL and dH
% p1 = 4; p2 = 10;
% dLH = zeros(size(fLH));
% dLH(1:p1-1) = dL(1:p1-1);
% dLH(p2+1:end) = dH(p2+1:end);
% dLH(p1:p2) = (dL(p1:p2)+dH(p1:p2))/2;
% importance_freq.dL = interp1(xLH, dL, xAI, 'linear', 'extrap');
% importance_freq.dH = interp1(xLH, dH, xAI, 'linear', 'extrap');
% importance_freq.dLH = interp1(xLH, dLH, xAI, 'linear'); % method3 for the integration of dL and dH
% p = find(importance_freq.dLH<0); importance_freq.dLH(p) = 0;

% find the intersection point of eL and eH and label it on the figure
% eL0 = eL;
% for i=2:length(eL)
%     eL0(i) = min(eL0(i-1), eL0(i));
% end
eL1 = interp1(xLH, eL, xAI, 'linear', 'extrap');
eH1 = interp1(xLH, eH, xAI, 'linear', 'extrap');
err = eL1 - eH1; 
err_product = err(1:end-1).*err(2:end); 
p = max(find(err_product<=0)); % the position where the sign of err changes
if abs(err(p))>abs(err(p+1))
    band_X = p+1;
else
    band_X = p;
end


%% calculate AI-grams for the utterance
snrs = [-12, -6, 0, 6, 12, 18];
talker = utter(1:4); cv = utter(6:end); utterance = ['s_' utter];

cd('./aigram_v4.2');
[AI_cube, s, speech_only, Fs] = creat_AI_cube(talker, cv, snrs, '16');
cd('..');

psnr = find(snrs==snr);

AIgram.cc = AI_cube(psnr).cc;
AIgram.ccgram = AI_cube(psnr).ccgram;
AIgram.time_axis = AI_cube(psnr).time_axis;
AIgram.freq_axis = AI_cube(psnr).freq_axis;

%---------------------------


%% calculate the time importance function
% align the time importance function with AIgram
% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number
delay_aigram = 736; % delay of the auditory filter introduced by AIgram 4.2

% find the shift in samples between the original speech and speech_only 
[shift, error] = speech_match(s, speech_only, Fs); 

delay_total = shift + delay_aigram;  % total delay of speech_only
t = t/100 + delay_total/Fs;

% align the CP of truncation data to the AIgram
cp_time.conds = cp_time.conds/100 + delay_total/Fs;

[nband, nframe] = size(AIgram.ccgram);

framesize = 0.0025; % seconds

pe_time.eT = eT;    % probability error of truncated speech
pe_time.t = t;      % truncation time

p1 = find(abs(AIgram.time_axis-t(1))==min(abs(AIgram.time_axis-t(1))));
p2 = find(abs(AIgram.time_axis-t(end))==min(abs(AIgram.time_axis-t(end))));

% Version 1: D_t(t) = s(t)*a(t); s(t) is the recognition score; a(t) the instantaneous AI 
importance_time.t = AIgram.time_axis;  % time scale
importance_time.dT = interp1(t, 1-eT, AIgram.time_axis, 'linear', 'extrap');
importance_time.dT(1:p1-1) = NaN;   % importance function calculated from truncation data
importance_time.dT(p2+1:end) = NaN;
importance_time.dAI = AIgram.cc/max(AIgram.cc);     % importance function calculated from instataneous AI
importance_time.dTAI = importance_time.dT.*importance_time.dAI;

% emin = 0.02;
% p = find(eT<emin); eT(p) = emin;
% ai_t = log10(eT)/log10(emin);
% dT = (ai_t(1:end-1)-ai_t(2:end))./(t(1:end-1)-t(2:end))/1000;
% dT = [dT; dT(end)]; % dT = dT/sum(dT);
% importance_time.t = t;
% %importance_time.dT = dT.*(1-eT);

fontsize = 10;
%% make plots for feature hunting
ax(1) = subplot(1,10,1), plot_CP_importance(cp_time); hold on
        %plot(t, 1-eT, '--r', 'Linewidth', 1); hold on;
        %plot(AIgram.time_axis, AIgram.cc, '--b', 'LineWidth', 1 ); hold on;
        %plot(importance_time.t, importance_time.dTAI, '-k', 'LineWidth', 2 );
        %plot(importance_time.t, importance_time.dT, '-k', 'LineWidth', 2 );
        ylim([-0.2, 1.1]); xlim([1, nframe]*framesize); grid off;
        set(gca, 'XTick', [0:0.05:round(nframe*framesize)], 'XTickLabel', [0:0.05:round(nframe*framesize)]*100,  'FontSize', fontsize); 
        set(gca, 'YTick', [0:0.2:1], 'FontSize', fontsize); %grid on;
        title('Time Importance','FontSize',fontsize+2);   
ax(2) = subplot(1,10,2), plot_CP_importance(cp_snr); hold on;
        %plot(snrs, 1-eSNR, '--b', 'Linewidth', 1); hold on;
        %plot(importance_snr.snrs, importance_snr.dSNR, '-k', 'LineWidth', 2 ); hold on;
        if (~isnan(SNR90))&&(SNR90<snrs(end))&&(SNR90>snrs(1)), plot(SNR90, 0.90, '*'); end
        xlim([-20, 26]);  ylim([-0.2, 1.1]); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'}, 'FontSize', fontsize); 
        set(gca, 'YTick', [-0.2:0.2:1], 'YTickLabel', '', 'FontSize', fontsize); %grid on;  
        title('Event Strength','FontSize',fontsize+2);
        text(-17, 0.85, 'c','FontSize',fontsize+2, 'EdgeColor','black');
ax(4) = subplot(1,10,4), 
        %plot(1-eL, band_AI, '--r', 1-eH, band_AI, ':b', 'Linewidth', 1); hold on;
        %h_legend = legend('LP', 'HP'); hold on; set(h_legend,'FontSize',fontsize-2);
        %plot(importance_freq.dL, importance_freq.bands, '-r', 'Linewidth', 1); hold on
        %plot(importance_freq.dH, importance_freq.bands, '-b', 'Linewidth', 1); hold on
        %plot(importance_freq.dLH, importance_freq.bands, '-k', 'Linewidth', 2); hold on       
        plot_CP_importance(cp_H); hold on
        plot_CP_importance(cp_L); hold on
        plot(1-eL1(band_X), band_X, 'p', 'Color', 'r', 'MarkerFacecolor', 'r','MarkerSize', 8);        
        %line([-0.5, 1.5], [band_X, band_X], 'Color', 'g', 'LineWidth', 2); hold on;
        %plot(importance_freq.dL, importance_freq.bands, ':r', importance_freq.dH, importance_freq.bands, ':b', 'Linewidth', 1); hold on;
        set(gca, 'XTick', [-0.2:0.2:1], 'FontSize', fontsize); xlim([-0.3, 1.1]);
        %set(gca, 'XTick', [floor(min(importance_freq.dLH)/0.2)*0.2:0.2:ceil(max(importance_freq.dLH)/0.2)*0.2]); 
        %xlim([floor(min(importance_freq.dLH)/0.2)*0.2, ceil(max(importance_freq.dLH)/0.2)*0.2]);
        set(gca,'YAxisLocation','l','YTick',linspace(1,length(fAI),11), 'YTickLabel', '', 'FontSize', fontsize);
        %set(gca,'YTickLabel',0.1*round(10*fAI(round(linspace(1,length(fAI),11)))/1000))
        ylim([1, 200]);   %grid on;
        xlabel('Frequency Importance','FontSize', fontsize+2);
        %ylabel('Frequency [kHz]','FontSize',fontsize)
        %title('Hi-Lo Data','FontSize',fontsize);
        %legend('L','H');
        text(-0.2, 180, 'd','FontSize',fontsize+2, 'EdgeColor','black');
        text(1-eL(11), band_AI(11), 'L', 'FontSize',fontsize);
        text(1-eH(2), band_AI(2), 'H', 'FontSize',fontsize);        
ax(3) = subplot(1,10,3), 
        plot_AI_gram(AIgram, utterance, snr); hold on; %grid on;

        start_time = min(AIgram.time_axis);
        end_time = max(AIgram.time_axis);
        [nband, nframe] = size(AIgram.ccgram);
        con_start_time = min(t); 
        con_end_time = max(t);
        vow_start_time = vow_start_time/100 + delay_total/Fs;
        
%         %%----Detect the event region based on the AIgram(snr*) and AIgram(snr*-6)
%         aigram1 = AI_cube(psnr).ccgram;
%         aigram2 = AI_cube(psnr-1).ccgram;        
%         
%         BW1 = im2bw(aigram1, 0.005);
%         BW2 = im2bw(aigram2, 0.005);
%         
%         M = importance_freq.dLH'*importance_time.dTAI;
%         p = find(M<0); M(p) = 0; p = find(isnan(M)); M(p) = 0;
%         N = BW1 - BW2;
%         
%         SE1 = strel('rectangle',[3 3]); % set the basic structure to be a rectangle        
%         SE2 = strel('rectangle',[5 5]); % set the basic structure to be a rectangle                
%         N1 = imerode(N,SE1); 
%         N2 = imdilate(N1,SE2); 
%                         
%         REG = M.*N2;
% 
%         contour(REG, '--b');
        %%-----------------------------------------------------------------

        con_start_col = round((con_start_time-start_time)/(end_time-start_time)*nframe);
        con_end_col = round((con_end_time-start_time)/(end_time-start_time)*nframe);
        vow_start_col = round((vow_start_time-start_time)/(end_time-start_time)*nframe);

        line([con_start_col con_start_col], [1 200], 'Color', 'b' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        line([con_end_col con_end_col], [1 200], 'Color', 'b' ,'Linewidth', 1, 'Linestyle', '-' );
        line([vow_start_col vow_start_col], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '-' );                  
   
    % cut the blank part of aigram
    ax(3) = subplot(1,10,3), xlim([con_start_col-5, vow_start_col+60]);
        %text(vow_start_col-6, 180, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',12);
        text(vow_start_col+57, 20, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',fontsize+2,...
             'HorizontalAlignment','right','EdgeColor','black','BackgroundColor',[1 1 1]);
        text(vow_start_col+55, 180, 'a','FontSize',fontsize+2, 'EdgeColor','black');
    ax(1) = subplot(1,10,1), xlim([(con_start_col-5)*0.0025, (vow_start_col+60)*0.0025]);
        text((vow_start_col+55)*0.0025, 0.85, 'b','FontSize',fontsize+2, 'EdgeColor','black');
        
%% generate the AI-grams under various snrs

    %% make plots for aigrams
    S = load('standard.mat');
    frequencies = S.frequencies;

    for i=1:length(snrs)

        AIgram.ccgram = AI_cube(i).ccgram;
        AIgram.time_axis = AI_cube(i).time_axis;
        AIgram.freq_axis = AI_cube(i).freq_axis;
    
        [nband, nframe] = size(AIgram.ccgram);

        stime = min(AIgram.time_axis);
        etime = max(AIgram.time_axis);
        
        con_stime = min(t); 
        con_etime = max(t);
        
        vow_stime = vow_start_time/100 + delay_total/Fs;

        con_scol = round((con_stime-stime)/(etime-stime)*nframe);
        con_ecol = round((con_etime-stime)/(etime-stime)*nframe);
        vow_scol = round((vow_stime-stime)/(etime-stime)*nframe);
        
        ax(i+4) = subplot(1, 4+length(snrs), i+4), 
        
        imagesc(AIgram.ccgram); hold on;
        colormap(flipud(gray));
        
        name = fix_underscore(utter);

        %if (i==1), text(con_scol+5, 184, fix_TSDZ(name, 1), 'FontSize',14); end

        %line([con_scol con_scol], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        %line([con_ecol con_ecol], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        line([vow_scol vow_scol], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '--' );                       

        if (i==1), 
            ylabel('Frequency [kHz]','FontSize',fontsize+2); 
            text(con_scol+5, 180, 'e','FontSize',fontsize+2, 'EdgeColor','black');
        end
        %xlabel('Time [cs]','FontSize',fontsize+2);
        
        set(gca,'YDir','normal')
        set(gca,'FontSize',fontsize)        
        set(gca,'YAxisLocation','l','YTick',linspace(1,nband,11),'YTickLabel', '', 'FontSize', fontsize);
        if (i==1), set(gca,'YTickLabel',0.1*round(10*frequencies(round(linspace(1,nband,11)))/1000)); end
        
        set(gca,'XTick',20*[1:floor(nframe/20)],'FontSize', fontsize);
        set(gca,'XTickLabel',5*[1:floor(nframe/20)], 'FontSize', fontsize);
        
        xlim([con_scol, con_ecol]);
        title([num2str(snrs(i)), ' dB'], 'FontSize', fontsize);
   
    end

%% set the positions of the subplots
%     set(ax(1), 'Position', [0.12, 0.68, 0.5, 0.22]);
%     set(ax(2), 'Position', [0.64, 0.68, 0.3, 0.22]);    
%     set(ax(3), 'Position', [0.12, 0.1, 0.5, 0.53]);
%     set(ax(4), 'Position', [0.64, 0.1, 0.3, 0.53]);

    set( ax(1), 'Position', [0.10, 12.5/16, 0.52, 2.5/16]);
    set( ax(2), 'Position', [0.64, 12.5/16, 0.31, 2.5/16]);    
    set( ax(3), 'Position', [0.10, 7/16, 0.52, 5/16]);
    set( ax(4), 'Position', [0.64, 7/16, 0.31, 5/16]);
    set( ax(5), 'Position', [0.10, 0.8/16, 0.15, 4.7/16]);
    set( ax(6), 'Position', [0.24, 0.8/16, 0.15, 4.7/16]);    
    set( ax(7), 'Position', [0.38, 0.8/16, 0.15, 4.7/16]);
    set( ax(8), 'Position', [0.52, 0.8/16, 0.15, 4.7/16]);
    set( ax(9), 'Position', [0.66, 0.8/16, 0.15, 4.7/16]);
    set(ax(10), 'Position', [0.80, 0.8/16, 0.15, 4.7/16]);    

