% This program calculate the time, frequency, and snr importance 
% functions based on TR07, HL07, and MN16R data.

function calc_importance_snr_patch_0924(SPDB, utter, vow_start_time)

cv = utter(6:end);

%% calculate the probability error of the distorted speech
% calculate Pe as a function of truncation time
data = query(SPDB, 'exp', 'TRUNC07', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'time');
eT = pe_tem.score;  % probability error of truncated utterance
t = pe_tem.conds;   % truncation time

% calculate Pe as a function of high-pass and low-pass cutoff frequencies
data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'HP');
pe_tem = calc_Pe(data, cv, utter, 'hpf');
eH = pe_tem.score;  % probability error of high-pass filtered speech
fH = pe_tem.conds;  % high-pass cutoff frequency

data = query(SPDB, 'exp', 'HPLPF07', 'utter', utter, 'comment', 'LP');
pe_tem = calc_Pe(data, cv, utter, 'lpf');
eL = pe_tem.score;  % probability error of low-pass filtered speech
fL = pe_tem.conds;  % low-pass cutoff frequency

% calculate Pe as a function of signal-to-noise ratio
data = query(SPDB, 'exp', 'MN16R', 'utter', utter);
pe_tem = calc_Pe(data, cv, utter, 'snr');
eSNR = pe_tem.score;
snrs = pe_tem.conds;
snrs(end) = 24;
 
%% calculate the importance function from MN16R data
emin = 0.02;
p = find(eSNR<emin); eSNR(p) = emin;
ai_snr = log10(eSNR)/log10(emin);
dSNR = (ai_snr(2:end)-ai_snr(1:end-1))./(snrs(2:end)-snrs(1:end-1));
dSNR = [dSNR(1); dSNR];
dSNR = dSNR/(sum(dSNR));    % normalize the coefficients
importance_snr.snrs = snrs;
importance_snr.dSNR = dSNR.*(1-eSNR);

%% find the snr where the dominant feature is still available
p = find(dSNR==max(dSNR(5:end-1)));
%snr = snrs(max(p)); 
snr = 12;

%% calculate frequency importance functions
% interplate eL and eH across all the frequencies
fLH = unique(sort([fL; fH]));
eL = interp1(fL, eL, fLH, 'nearest', 'extrap');
eH = interp1(fH, eH, fLH, 'nearest', 'extrap');

eL(1) = eL(1) + (15/16-eL(1))/2;   % estimate information between 200Hz and 250Hz
eH(end) = 15/16;  % assuming no information above 8kHz

% get rid of eL and eH that are smaller than chance level
emin = 0.02;
p = find(eL<emin); eL(p) = emin;
p = find(eH<emin); eH(p) = emin;

% find the distance of the cutoff frequencies on BM
x1 = greenwood_inverse(184.4644);  % lowest frequency covered in AIgram
x2 = greenwood_inverse(7397.3);    % highest frequency covered in AIgram
xAI = linspace( x1, x2, 200);      % distances of 200 bands in AI-gram
fAI = greenwood_inverse(xAI, 'dist');

% convert the cut-off frequecies into auditory filter band number
xLH = greenwood_inverse(fLH);      % distances on BM that corresponds to fL and fH
for i=1:length(xLH)
    temp = abs(xAI-xLH(i));
    band_AI(i) = find(temp==min(temp));              % the AI band no corresponds to the cut-off frequency
end

% calculate the frequency importance function
dL = NaN(1, length(fLH)); 
dH = NaN(1, length(fLH)); 
dL(2:end) = log10(eL(2:end))/log10(emin) - log10(eL(1:end-1))/log10(emin);    % lpf weights ekL
dH(1:end-1) = log10(eH(1:end-1))/log10(emin) - log10(eH(2:end))/log10(emin);  % hpf weights ekH
dL(1) =  dH(1);  dH(end) =  dL(end);
%dL = dL/sum(dL); dH = dH/sum(dH);    % normalize the importance function
dL = dL.*(1-eL'); dH = dH.*(1-eH');

% save the data of HL07 in a struct
pe_freq.eL = eL; 
pe_freq.eH = eH;
pe_freq.dL = dL; 
pe_freq.dH = dH;
pe_freq.f = fLH;
pe_freq.x = xLH;
pe_freq.bands = band_AI;

% find the intersection point of eL and eH and label it on the figure
eL1 = interp1(xLH, eL, xAI, 'linear', 'extrap');
eH1 = interp1(xLH, eH, xAI, 'linear', 'extrap');
err = eL1 - eH1; 
err_product = err(1:end-1).*err(2:end); 
p = max(find(err_product<=0)); % the position where the sign of err changes
if abs(err(p))>abs(err(p+1))
    band_X = p+1;
else
    band_X = p;
end

% find the point for the merge of dL and dH
band_merge = find(abs((xAI-xLH(end-3)))==min(abs((xAI-xLH(end-3)))));

% interplate the frequency importance function
importance_freq.dL = interp1(xLH, dL, xAI, 'linear', 'extrap');
importance_freq.dH = interp1(xLH, dH, xAI, 'linear', 'extrap');
%importance_freq.dLH = (importance_freq.dL+importance_freq.dH)/2;
wL = ones(size(importance_freq.dL)); wL([band_merge+1:end]) = 0; wL([band_merge-10:band_merge+10]) = [1:-0.05:0]; 
wH = ones(size(importance_freq.dH)); wH([1:band_merge]) = 0;     wH([band_merge-10:band_merge+10]) = [0:0.05:1];
importance_freq.dLH = importance_freq.dL.*wL + importance_freq.dH.*wH;
importance_freq.x = xAI;
importance_freq.f = fAI;
importance_freq.bands = [1:length(xAI)];



%% calculate AI-gram for the utterance
% ---------------------------
utterance = ['s_' utter];
    
[s, Fs, bits]= wavread(['./wavs/' utterance '.wav']);
speech_data.fs = Fs;
speech_data.type = 'speechonly';
speech_data.snr = 0;
speech_data.signal = s;
    
cd('./aigram_v4.2');
        
filterbank = 'erbaligned';
%%1. Prepare the speech and generate the noise at 0 dB SNR like this:            
[ speech_in, speech_only, noise_only, noise_std ] = prepare_speech_mn16r( speech_data );  
%%2. Compute the auditory representation of the speech-only signal like this:
[ level3, rate ] = eval([ 'bm_filter_' filterbank '( speech_only, [1:200], ''speech'' , 0, 0 )' ]);
%%3. Compute the noise statistics:
[ E_ni2, E_ni ] = compute_noise_stats('mn16r', filterbank, std(noise_only) );       
%%4. Scale the noise statistics to the appropriate SNR:
E_ni2_scaled = E_ni2 * 10^(-snr/10);
E_ni_scaled = E_ni * 10^(-snr/20);            
%%5. Make AI-gram out of existing information:
[aigram, cc_d, time_axis, freq_axis ] = form_image_ai0607c( level3, 'mn16r', filterbank, E_ni2_scaled, E_ni_scaled);     
        
cd('..');
    
AIgram.cc = cc_d;
AIgram.ccgram = aigram;
AIgram.time_axis = time_axis;
AIgram.freq_axis = freq_axis;
%---------------------------

%% calculate the time importance function
% align the time importance function with AIgram
% This number only works when Fs = 16000, for other Fs re-calibration is
% needed to get this number
delay_aigram = 736; % delay of the auditory filter introduced by AIgram 4.2

% find the shift in samples between the original speech and speech_only 
[shift, error] = speech_match(s, speech_only, Fs); 

delay_total = shift + delay_aigram;  % total delay of speech_only
t = t/100 + delay_total/Fs;

[nband, nframe] = size(AIgram.ccgram);

framesize = 0.0025; % seconds

pe_time.eT = eT;    % probability error of truncated speech
pe_time.t = t;      % truncation time

p1 = find(abs(time_axis-t(1))==min(abs(time_axis-t(1))));
p2 = find(abs(time_axis-t(end))==min(abs(time_axis-t(end))));

% Version 1: D_t(t) = s(t)*a(t); s(t) is the recognition score; a(t) the instantaneous AI 
% importance_time.t = time_axis;  % time scale
% importance_time.dT = interp1(t, 1-eT, time_axis, 'linear', 'extrap');
% importance_time.dT(1:p1-1) = NaN;   % importance function calculated from truncation data
% importance_time.dT(p2+1:end) = NaN;
% importance_time.dAI = cc_d/max(cc_d);     % importance function calculated from instataneous AI
% importance_time.dTAI = importance_time.dT.*importance_time.dAI;

emin = 0.02;
p = find(eT<emin); eT(p) = emin;
ai_t = log10(eT)/log10(emin);
dT = (ai_t(1:end-1)-ai_t(2:end))./(t(1:end-1)-t(2:end))/1000;
dT = [dT; dT(end)]; dT = dT/sum(dT);
importance_time.t = t;
importance_time.dT = dT.*(1-eT);

%% make plots for feature hunting
ax(1) = subplot(221),
        plot(t, 1-eT, '--r', 'Linewidth', 1); hold on;
        %plot(time_axis, AIgram.cc, '--b', 'LineWidth', 1 ); hold on;
        %plot(importance_time.t, importance_time.dTAI, '-k', 'LineWidth', 2 );
        plot(importance_time.t, importance_time.dT, '-k', 'LineWidth', 2 );
        ylim([-0.2, 1.02]); xlim([1, nframe]*framesize); grid off;
        set(gca, 'XTick', [0:0.1:round(nframe*framesize)], 'XTickLabel', '', 'FontSize', 9); 
        set(gca, 'YTick', [-0.2:0.2:1]); %grid on;
        title('Time Importance','FontSize',12);        
ax(2) = subplot(222), 
        plot(snrs, 1-eSNR, '--b', 'Linewidth', 1); hold on;
        plot(importance_snr.snrs, importance_snr.dSNR, '-k', 'LineWidth', 2 );
        xlim([-20, 26]);  ylim([-0.2, 1.02]); 
        set(gca, 'XTick', [-18:6:24], 'XTickLabel', {'-18';'-12';'-6';'0';'6';'12';'18';'Q'}, 'FontSize', 9); 
        set(gca, 'YTick', [-0.2:0.2:1], 'YTickLabel', '' ); %grid on;  
        title('Event Strength','FontSize',12);
ax(4) = subplot(224), plot(1-eL, band_AI, '--r', 1-eH, band_AI, '--b', 'Linewidth', 1); hold on;
        %line([-0.5, 1.5], [band_X, band_X], 'Color', 'g', 'LineWidth', 2); hold on;
        %plot(importance_freq.dL, importance_freq.bands, ':r', importance_freq.dH, importance_freq.bands, ':b', 'Linewidth', 1); hold on;
        plot(importance_freq.dLH, importance_freq.bands, '-k', 'Linewidth', 2);
        set(gca, 'XTick', [-0.4:0.2:1]); xlim([-0.4, 1.1]);
        %set(gca, 'XTick', [floor(min(importance_freq.dLH)/0.2)*0.2:0.2:ceil(max(importance_freq.dLH)/0.2)*0.2]); 
        %xlim([floor(min(importance_freq.dLH)/0.2)*0.2, ceil(max(importance_freq.dLH)/0.2)*0.2]);
        set(gca,'YAxisLocation','l','YTick',linspace(1,length(fAI),11), 'YTickLabel', '', 'FontSize', 8);
        %set(gca,'YTickLabel',0.1*round(10*fAI(round(linspace(1,length(fAI),11)))/1000))
        ylim([1, 200]);   %grid on;
        xlabel('Frequency Importance','FontSize', 12);
        %ylabel('Frequency [kHz]','FontSize',12)
        %title('Hi-Lo Data','FontSize',12);
        %legend('L','H');
        text(1-eL(11), band_AI(11), 'L', 'FontSize',12, 'EdgeColor','black');
        text(1-eH(2), band_AI(2), 'H', 'FontSize',12, 'EdgeColor','black');        
ax(3) = subplot(223), 
        plot_AI_gram(AIgram, utterance, snr); hold on; %grid on;

        start_time = min(AIgram.time_axis);
        end_time = max(AIgram.time_axis);
        [nband, nframe] = size(AIgram.ccgram);
        con_start_time = min(t); 
        con_end_time = max(t);
        vow_start_time = vow_start_time/100 + delay_total/Fs;
        
% %         %--------------------------------------------------------------
% %         % plot the contour of importance function
% %         p1 = find(abs(AIgram.time_axis-t(1))==min(abs(AIgram.time_axis-t(1))));
% %         p2 = find(abs(AIgram.time_axis-t(end))==min(abs(AIgram.time_axis-t(end))));
% %         
% %         importance_time.dT = interp1(t, importance_time.dT, time_axis, 'nearest', 'extrap');
% % %         p = find(importance_time.dT<0.25); importance_time.dT(p) = 0;
% % %         q = find(importance_freq.dLH<0.2); importance_freq.dLH(q) = 0;
% %         z = importance_freq.dLH'*importance_time.dT;
% %         z = z.*AIgram.ccgram;
% %         % contour(x,y,z, '-g');
% %         
% %         % Morphological method
% %         p = find(z>0); level = 2*std(z(p));
% %         BW = im2bw(z, level); 
% %         SE = strel('rectangle',[5 3]); % set the basic structure to be a rectangle
% %         BW1 = imerode(BW,SE); 
% %         BW2 = imdilate(BW1,SE); 
% %         contour(BW2*1, 1, '--b');
% %         %C = contourc(BW2*1); 
% %         %C = C(:,2:end);
% %         %patch(C(1,:), C(2,:), [0 0.5 0], 'FaceAlpha', 0.5, 'EdgeColor', [0 0.5 0]); 
% % 
% %         %--------------------------------------------------------------

        con_start_col = round((con_start_time-start_time)/(end_time-start_time)*nframe);
        con_end_col = round((con_end_time-start_time)/(end_time-start_time)*nframe);
        vow_start_col = round((vow_start_time-start_time)/(end_time-start_time)*nframe);

        line([con_start_col con_start_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' ); hold on;
        line([con_end_col con_end_col], [1 200], 'Color', 'r' ,'Linewidth', 1, 'Linestyle', '-' );
        line([vow_start_col vow_start_col], [1 200], 'Color', 'g' ,'Linewidth', 1, 'Linestyle', '-' );                  
   
    % cut the blank part of aigram
    ax(3) = subplot(223), xlim([con_start_col-5, vow_start_col+60]);
                          %text(vow_start_col-6, 180, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',12);
                          text(vow_start_col+55, 180, fix_TSDZ([fix_underscore(utter) '@' num2str(snr) 'dB'], 1), 'FontSize',14,...
                              'HorizontalAlignment','right','EdgeColor','black','BackgroundColor',[1 1 1]);
    ax(1) = subplot(221), xlim([(con_start_col-5)*0.0025, (vow_start_col+60)*0.0025]);
        
    set(ax(1), 'Position', [0.12, 0.68, 0.5, 0.22]);
    set(ax(2), 'Position', [0.64, 0.68, 0.3, 0.22]);    
    set(ax(3), 'Position', [0.12, 0.1, 0.5, 0.53]);
    set(ax(4), 'Position', [0.64, 0.1, 0.3, 0.53]);
   
% % adjust the position and size of the axes
% set(ax(1), 'Position', [0.1, 0.65, 0.6, 0.22]);
% set(ax(2), 'Position', [0.74, 0.65, 0.22, 0.22]);    
% set(ax(3), 'Position', [0.1, 0.1, 0.6, 0.5]);
% set(ax(4), 'Position', [0.74, 0.1, 0.22, 0.5]);
